// ESLint has trouble recognizing the presence of `RequestInfo` and `RequestInit` despite TypeScript resolving their type definitions in VS Code.
/* eslint-disable no-undef, no-unused-vars */

// TODO: rename this file to `index.ts` in order to get the types properly checked.

export type MakeRequestOptions = {
  url: RequestInfo
  init?: Omit<RequestInit, 'signal'>

  /**
   * Causes the request to fail after a number of seconds.
   */
  timeoutInSeconds?: number

  /**
   * Data for the error to throw in the case of a network error (e.g. when the server cannot be reached) or a request error (e.g. the request cannot be authenticated).
   */
  networkErrorMessage: string
  networkErrorCode: string
  requestErrorMessage: string
  requestErrorCode: string

  deliveryMethodId: string | undefined
}

export type BaseOptions = {
  mountElement: HTMLElement
  renderDate?: Date
  checkoutConfiguration?: CheckoutConfiguration
  deliveryZone?: DeliveryZone
  showCloseButton?: boolean
  state?: Record<string, any>
  locale: string
  localeMessages?: Record<string, string>
  dispatcher?: Element | Window | Document

  /**
   * Origin for requests to the service points API. Necessary option for package consumers that want to provide a different URL for local development environments (like for the use of the service point picker in the platform’s new shipment form).
   */
  checkoutApiOrigin?: string
}

export type NominatedDayDeliveryOptions = BaseOptions & {
  deliveryMethod: NominatedDayDeliveryMethod
  shippingData?: ShippingData
}

export type ServicePointDeliveryOptions = BaseOptions & {
  deliveryMethod: ServicePointDeliveryMethod
  shippingData: ShippingData
}

export type CheckoutPluginUiOptions = ServicePointDeliveryOptions | NominatedDayDeliveryOptions

export type CheckoutConfiguration = {
  id: string

  /**
   * A JSON Web Token encoding either an integration or a user ID. Will be used to authenticate with all checkout APIs.
   */
  access_token: string
}

export type DeliveryZone = {
  id: string
  country: DeliveryZoneCountry
}

export type DeliveryZoneCountry = {
  iso_2: string
  name: string
}

/**
 * A delivery method holds the main information about a merchant’s custom shipping options.
 * A delivery method belongs to a delivery zone which in turn belongs to a checkout configuration.
 *
 * @see {DeliveryZone}
 * @see {CheckoutConfiguration}
 */
export type NominatedDayDeliveryMethod = {
  id: string
  delivery_method_type: DeliveryMethodType
  external_title: string
  enabled: boolean
  show_carrier_information_in_checkout: boolean
  shipping_product: DeliveryMethodShippingProduct
  carrier: DeliveryMethodCarrier
  parcel_handover_days: DeliveryMethodParcelHandoverDays

  /**
   * A time zone name according to the IANA time zone database (e.g. `'Europe/Amsterdam'`).
   */
  time_zone_name: string

  holidays?: DeliveryMethodHolidays | DeliveryMethodHoliday[]
  shipping_rate_data?: DeliveryMethodShippingRateData
}

export type ServicePointDeliveryMethod = {
  id: string
  delivery_method_type: 'service_point_delivery'
  external_title: string
  enabled: boolean
  carriers: DeliveryMethodCarrier[]

  /**
   * A time zone name according to the IANA time zone database (e.g. `'Europe/Amsterdam'`).
   */
  time_zone_name: string

  shipping_rate_data?: DeliveryMethodServicePointShippingRateData
  service_point_data: DeliveryMethodServicePointData
}

type DeliveryMethodShippingProduct = {
  code: string
  name: string
  selected_functionalities: DeliveryMethodSelectedFunctionalities
  carrier_delivery_days: DeliveryMethodCarrierDeliveryDays
  lead_time_hours: number | null
  lead_time_hours_override?: number | null
}

type DeliveryMethodSelectedFunctionalities = Record<string, DeliveryMethodSelectedFunctionalityValues>

type DeliveryMethodSelectedFunctionalityValues = null | boolean | number | string | Array<null | boolean | number | string>

export type DeliveryMethodCarrier = {
  name: string
  code: string
}

/**
 * The carrier delivery days hold the information on which week days a carrier is delivering for a specific shipping product configuration.
 *
 * If a delivery day is `null`, it cannot be selected by a consumer.
 * If it’s an object, it will be shown to clients.
 */
export type DeliveryMethodCarrierDeliveryDays = Record<WeekDay, {} | null>

export type DeliveryMethodParcelHandoverDays = Record<WeekDay, DeliveryMethodParcelHandoverDayData | null>

export type DeliveryMethodHolidays = {
  enabled: boolean
  holiday_dates: DeliveryMethodHoliday[]
}

export type DeliveryMethodHoliday = {
  title: string
  from_date: string
  to_date: string
  recurring: boolean

  /**
   * The frequency of the holiday occurring. Is `null` if `recurring` is `false`.
   */
  frequency: null | 'yearly'
}

export type DeliveryMethodShippingRateData = {
  enabled: boolean
  currency: string
  free_shipping: {
    enabled: boolean
    from_amount: string
  }
  shipping_rates: DeliveryMethodShippingRate[]
}

export type DeliveryMethodShippingRate = {
  enabled: boolean
  is_default: boolean
  min_weight: number // gram
  max_weight: number // gram
  rate: string
}

export type DeliveryMethodServicePointShippingRateData = {
  enabled: boolean
  currency: UserCurrency
  free_shipping: {
    enabled: boolean
    from_amount: string
  }
  shipping_rates: DeliveryMethodServicePointShippingRate[]
}

export type DeliveryMethodServicePointShippingRate = {
  enabled: boolean
  rate: string | null
  carrier_code: string
}

export type WeekDay = 'monday' | 'tuesday' | 'wednesday' | 'thursday' | 'friday' | 'saturday' | 'sunday'

/**
 * A delivery method has a type that identifies the kind of custom shipping option it specifies.
 *
 * @see {DeliveryMethod}
 */
export type DeliveryMethodType = 'service_point_delivery' | 'nominated_day_delivery'

type DeliveryMethodParcelHandoverDayData = {
  /**
   * Indicates on which days a merchant is handing over parcels to the carrier.
   * A parcel handover is either an arranged pickup of the parcels by the carrier
   * or the merchant handing parcels in a parcel shop.
   *
   * This data along with a shipping product configuration’s lead time influences
   * which delivery dates are offered to the user in the checkout plugin UI.
   */
  enabled: boolean

  /**
   * The hour value of the cut-off time in 24 hour format.
   */
  cut_off_time_hours: number

  /**
   * The minute value of the cut-off time.
   */
  cut_off_time_minutes: number
}

/**
 * A delivery date object holds the data for rendering a delivery date in the UI.
 */
export type DeliveryDate = {
  /**
   * The name of the form control.
   * Important for grouped controls like radio buttons and checkboxes.
   */
  name: string

  /**
   * The ID of the form control.
   * Used to identify one specific element when re-rendering.
   */
  id: string

  /**
   * The delivery date formatted according to ISO 8601.
   */
  isoDeliveryDate: string

  /**
   * The full formatted delivery date. Based on `options.locale`.
   */
  formattedDeliveryDate: string

  /**
   * The parcel handover date formatted according to ISO 8601.
   */
  isoParcelHandoverDate: string

  /**
   * The full formatted parcel handover date. Based on `options.locale`.
   */
  formattedParcelHandoverDate: string

  /**
   * The week day and day. Based on `options.locale`.
   */
  weekDay: string

  /**
   * The month’s name. Based on `options.locale`.
   */
  month: string
  shortMonth: string

  /**
   * Whether to add the `checked` attribute.
   */
  isChecked: boolean | undefined
}

export type ShippingOptionChangeEventData<Type> = {
  delivery_method_type: DeliveryMethodType
  delivery_method_data: DeliveryMethodTypeDetailMap[Type]
}

type DeliveryMethodTypeDetailMap = {
  service_point_delivery: ServicePointDeliveryData
  nominated_day_delivery: NominatedDayDeliveryData
}

export type ServicePointDeliveryData = {
  service_point: ServicePoint
  service_point_id: number
  post_number: string
  carrier_code: string
  formatted_address: string
}

export type NominatedDayDeliveryData = {
  parcel_handover_date: string
  formatted_parcel_handover_date: string
  delivery_date: string
  formatted_delivery_date: string
}

/**
 * A service point object.
 */
export type ServicePoint = {
  /**
   * Carrier code
   */
  carrier: string
  city: string
  code: string
  country: string
  distance?: number
  email: string
  extra_data: {
    shop_type?: 'packStation'
    pudo_id?: string
    shipper_shop_id?: string
    partner_name?: string
    sales_channel?: string
    terminal_type?: string
    retail_network_id?: string
  }
  formatted_opening_times: {
    0: string[]
    1: string[]
    2: string[]
    3: string[]
    4: string[]
    5: string[]
    6: string[]
  }
  homepage: string
  house_number: string
  id: number
  is_active: boolean
  latitude: string
  longitude: string
  name: string
  open_tomorrow: boolean
  open_upcoming_week: boolean
  phone: string
  postal_code: string
  shop_type: string | null
  street: string
}

/**
 * An object of map coordinates.
 *
 * Matches {@link mapboxgl.LngLatLike}
 */
export type ServicePointCoordinate = {
  lng: number
  lat: number
}

export type DeliveryMethodServicePointData = {
  /**
   * Access token used for communication with the service points API. Typically a JSON Web Token that encodes either an integration ID or a user ID.
   */
  api_key: string

  /**
   * Access token for the Mapbox APIs.
   */
  mapbox_access_token?: string

  /**
   * Destination country for which service points should be located.
   */
  country_iso_2: string
}

/**
 * Shipping data is any data the buyer is supplying during a checkout process (e.g. by filling our the form fields for their shipping address).
 */
export type ShippingData = {
  /**
   * ID of a pre-selected service point.
   */
  service_point_id?: number

  /**
   * A PO box number. Used to populate the PO box number text field on all service point entries that require it.
   */
  post_number?: string

  /**
   * Postal code of the shipping address for which service points should be located.
   */
  postal_code?: string

  /**
   * City of the shipping address for which service points should be located.
   */
  city?: string

  /**
   * The total price of a cart’s items (found by summing up all cart item prices multiplied by their quantities).
   */
  cart_price?: number
  cart_price_currency?: string

  /**
   * The total weight of a cart’s items (found by summing up all cart item weights multiplied by their quantities).
   */
  cart_weight?: number
  cart_weight_unit?: 'gram'

  /**
   * A list of item dimensions that apply to the products in a buyer’s cart.
   *
   * **Example**: The dimensions for two cart items. The first measures 100 cm × 10 cm × 10 cm; the second measures 10 cm × 13 cm × 18 cm.
   *
   * ```json
   * {
   *   "cart_item_dimensions": [[100, 10, 10], [10, 13, 18]],
   *   "cart_item_dimensions_unit": "cm"
   * }
   * ```
   */
  cart_item_dimensions?: Array<number[]>
  cart_item_dimensions_unit?: ServicePointsApiDimensionUnit

  /**
   * A unique order ID that we send to the Sendcloud platform via the service points check-and-store endpoint in order to associate the selected service point with an order.
   *
   * This is necessary because some shop systems (PrestaShop, Lightspeed) don’t allow us to store meta data on orders as they will be imported by Sendcloud.
   */
  store_order_id?: string

  /**
   * A set of unique order identifiers we send to the Sendcloud platform via the service points check-and-store endpoint in order to associate the selected service point with an order.
   *
   * This is necessary because Shopify doesn’t allow us to store meta data on orders as they will be imported by Sendcloud.
   */
  checkout_shipping_method_id?: string
  checkout_shipping_method_name?: string
}

export type ServicePointEntry = {
  id: string
  name: string
  servicePointId: number
  servicePointName: string
  formattedPrice: string,
  coordinate: ServicePointCoordinate
  street: string
  houseNumber: string
  postalCode: string
  city: string
  carrierCode: string
  isOpenInTheUpcomingWeek: boolean
  isOpenTomorrow: boolean
  isSelected: boolean
  type: 'packStation' | 'servicePoint'
  formattedDistance: string | null
  openingTimes: Record<string, string[]>
  openingTimeRangesForTomorrow: string[]
  formattedAddress: string
  poBoxNumberId: string
  poBoxNumberInputValue: string
}

export type ServicePointsApiBaseParameters = {
  /**
   * A JSON Web Token encoding either an `id` property representing a Sendcloud user ID or an `iid` property representing a Sendcloud integration ID. You can verify this on https://jwt.io.
   */
  access_token: string
}

export type ServicePointsApiParameters = ServicePointsApiBaseParameters & {
  /**
   * Country ISO 2 code (e.g. `'NL'`)
   */
  country: string

  /**
   * A string of carrier codes (e.g. `'ups'`). Multiple carrier codes can be provided separated by commas (e.g. `'postnl,ups'`).
   */
  carrier?: string

  /**
   * A set of geographical data points used to retrieve a fitting list of service points.
   *
   * This also determines the value of the `distance` field provided on the resulting service points.
   */
  latitude: string
  longitude: string
  ne_latitude: string
  ne_longitude: string
  sw_latitude: string
  sw_longitude: string

  /**
   * Opts out of some specific and undesirable validation logic on the service points collection resource endpoint.
   *
   * - It allows us to query for service points without service points being activated as a feature in the integration settings.
   * - It allows us to query for service point carriers that haven’t been activated in the integration settings for service points.
   */
  ignore_integration_settings?: 'true' | 'false'
}

export type ServicePointsApiCheckAndStoreParameters = ServicePointsApiBaseParameters & {
  /**
   * The service point ID (stringified).
   */
  service_point: string

  /**
   * The entered PO box number. Expected to be the empty string if not provided.
   */
  to_post_number: string

  /**
   * Indicates whether the availability and opening times of the service point should be verified.
   */
  check_availability: 'true' | 'false'

  /**
   * Provides a unique reference to an order so that a selected service point can be stored in the Sendcloud platform in case of a shop system not allowing us to store it with the order’s meta data (e.g. PrestaShop, Lightspeed).
   *
   * Expected to be the empty string if not provided.
   */
  store_order_id: string

  /**
   * Stored alongside a service point. Unclear what it’s for.
   */
  checkout_shippingmethod_id: string

  /**
   * Stored alongside a service point. Unclear what it’s for.
   */
  checkout_shippingmethod_name: string
}

/**
 * The response body shape of the service point check-and-store API endpoint.
 */
export type ServicePointsApiCheckAndStoreResponse = {
  /**
   * The status of the service point:
   *
   * - 0: the service point **is not** available.
   * - 1: the service point **is** available.
   * - 2: the service point’s data has changed. This generally happens when its opening times changed.
   */
  availability: ServicePointNotAvailable | ServicePointAvailable | ServicePointChanged

  /**
   * Indicates whether the service point selection was successfully stored in the Sendcloud platform. Not present if the service point wasn’t stored.
   */
  stored?: true

  /**
   * The up-to-date service point associated with the service point ID provided in the request.
   */
  service_point: ServicePoint
}

export type ServicePointsApiItemsFitInShipmentParameters = ServicePointsApiBaseParameters

export type ServicePointsApiItemsFitInShipmentData = {
  carrier_codes: string[]
  dimensions: Array<number[]>
  unit: ServicePointsApiDimensionUnit
}

export type ServicePointsApiDimensionUnit = 'mm' | 'cm' | 'dm' | 'm' | 'in' | 'yd'

export type ServicePointsApiItemsFitInShipmentResponse = {
  fitting_carrier_codes: string[]
}

export type ServicePointNotAvailable = 0
export type ServicePointAvailable = 1
export type ServicePointChanged = 2

export type ServicePointLocationSearchResult = {
  id: string
  shortPlaceName: string
  longPlaceName: string
  coordinate: ServicePointCoordinate
}

export type StaticData = {
  CHECKOUT_PLUGIN_UI_BASE_URL: string
  MAPBOX_ACCESS_TOKEN: string
}

declare const renderScShippingOption: (options: CheckoutPluginUiOptions) => Promise<() => void>

declare global {
  interface Window {
    renderScShippingOptionModule?: Promise<{ renderScShippingOption: typeof renderScShippingOption }>
    renderScShippingOption?: typeof renderScShippingOption
  }
}

// The type that’s visible when importing the named export of the package
export { renderScShippingOption }
